import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import java.net.*;
import javax.sound.sampled.AudioFileFormat;
import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.Clip;
import javax.sound.sampled.DataLine;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.UnsupportedAudioFileException;

/**
 * Interfaz grfica para ChatClient
 *
 * @version 0.1_Beta 01-11-2005
 * @author Carlos Silva Ltty
 */
public class ChatViewer {
    protected Socket socket;
    /**
     * Contructor de la clase ChatViewer
	 * 
     * @param socket Socket de la conexin del cliente.
     * @param user Nombre del usuario.
     */
    public ChatViewer(Socket socket, String user) {
        this.socket = socket;
        JFrame frame = new ChatViewerFrame(socket, user);
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        frame.show();
    }
    
}

/**
 * Una interfaz grfica para mostrar los mensajes de los dems usuarios.
 */
class ChatViewerFrame extends JFrame implements Runnable {
    
    private static final int DEFAULT_WIDTH = 480;
    private static final int DEFAULT_HEIGHT = 640;
    protected Socket socket;
    protected DataInputStream i;
    protected DataOutputStream o;
    protected Thread thread;
    TextField input = new TextField();
    JEditorPane output = new JEditorPane();
    JTextArea users = new JTextArea();
    JSplitPane jSplitPane1 = new JSplitPane();
    JSplitPane jSplitPane2 = new JSplitPane();
    JSplitPane jSplitPane3 = new JSplitPane();
    JSplitPane jSplitPane4 = new JSplitPane();
    JPanel jPanel1 = new JPanel();
    JPanel jPanel2 = new JPanel();
    JPanel jPanel3 = new JPanel();
    JLabel jLabel1 = new JLabel();
    JLabel jLabel2 = new JLabel();
    JButton jButton1 = new JButton();
    JScrollPane jScrollPane1 = new JScrollPane(output, 20, 30);
    JScrollPane jScrollPane2 = new JScrollPane(users, 20, 30);
    File aux;
    RandomAccessFile archivo;
    String datos;
 
 /**
  * Constructor de clase ChatViewerFrame.
  * 
  * @param socket Socket del cliente.
  * @user nombre de usuario.
  */
    
    public ChatViewerFrame(Socket socket, String user) {
        final String userName = user;
        this.socket = socket;
        try {
            i = new DataInputStream(new BufferedInputStream(socket.getInputStream()));
            o = new DataOutputStream(new BufferedOutputStream(socket.getOutputStream()));
        } catch (Exception e) {}
        setTitle("J@Zz_Ch@t");
        setSize(DEFAULT_WIDTH, DEFAULT_HEIGHT);
        
        try{
            aux=new File(user + ".txt");
            aux.deleteOnExit();
            archivo= new RandomAccessFile(aux, "rw");
            
        } catch (IOException excepcionES){}
        
        JLabel label = new JLabel();
        Container contentPane = getContentPane();
        contentPane.add(label);
        
        JMenuBar menuBar = new JMenuBar();
        setJMenuBar(menuBar);
        
        JMenu menuFile = new JMenu("File");
        menuBar.add(menuFile);
        
        JMenuItem exitItem = new JMenuItem("Exit");
        menuFile.add(exitItem);
        exitItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event) {
                try{
                    archivo.close();
                } catch (IOException x){}
                System.exit(0);
            }
        });
        
        JMenu menuConfig = new JMenu("Config");
        menuBar.add(menuConfig);
        
        JMenuItem colorItem = new JMenuItem("Color");
        menuConfig.add(colorItem);
        colorItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event) {
                JFrame frame2 = new JFrame();
                Color colornuevo= JColorChooser.showDialog(frame2, "Seleccione un color para el texto", input.getForeground());
                input.setForeground(colornuevo);
            }
        });
        
        JMenu menuHelp = new JMenu("Help");
        menuBar.add(menuHelp);
        
        JMenuItem aboutItem = new JMenuItem("About");
        menuHelp.add(aboutItem);
        aboutItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event) {
                JOptionPane.showMessageDialog(new JFrame(), "Carlos Silva Ltty\nAndrs Cern Corrales.\nVersion 0.1_BETA 30/10/2005");
            }
        });
        
        jSplitPane1.setOrientation(JSplitPane.VERTICAL_SPLIT);
        jSplitPane4.setOrientation(JSplitPane.VERTICAL_SPLIT);
        users.setEditable(false);
        output.setEditable(false);
        output.setContentType("text/html");
        input.setForeground(Color.black);
        input.setBackground(Color.white);
        jLabel1.setText("j@ZzLuttYChat");
        jLabel1.setFont(new Font("Dialog", 1, 40));
        jButton1.setText("Send");
        jLabel2.setText("USERS @N_LINE\n");
        jPanel3.add(jButton1, null);
        jPanel2.add(jLabel2, null);
        jPanel1.add(jLabel1, null);
        
        contentPane.add(jSplitPane1, BorderLayout.CENTER);
        contentPane.add(jSplitPane2, BorderLayout.SOUTH);
        jSplitPane1.add(jSplitPane3, JSplitPane.BOTTOM);
        jSplitPane3.add(jSplitPane4, JSplitPane.LEFT);
        jSplitPane1.add(jPanel1, JSplitPane.TOP);
        jSplitPane4.add(jPanel2, JSplitPane.TOP);
        jSplitPane2.add(jPanel3, JSplitPane.LEFT);
        jSplitPane2.add(input, JSplitPane.RIGHT);
        jSplitPane3.add(jScrollPane1, JSplitPane.BOTTOM);
        jSplitPane4.add(jScrollPane2, JSplitPane.BOTTOM);
        
        //Accin tanto al presionar el botn SEND
        
        jButton1.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                try {
                    if (input.getText().length() > 0) {
                        o.writeUTF(input.getForeground().getRGB() + "<" + userName + ">  " + input.getText());
                        o.flush();
                    }
                } catch (Exception ex) {}
                input.setText("");
            }
        });
        
        // Accin al presionar enter en el campo input.
        
        input.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                try {
                    if (input.getText().length() > 0) {
                        o.writeUTF(input.getForeground().getRGB() + "<" + userName + ">  " + input.getText()); //se transmite el color.
                        o.flush();
                    }
                } catch (Exception ex) {}
                input.setText("");
            }
        });
        
        thread = new Thread(this);
        thread.start();
    }
 
    /**
     * Mtodo que permite la reproduccin de un sonido.
	 * 
     * @param sonidito Nombre del sonido a reproducir.
     */
    
    public void sonido(String sonidito){
        File sf=new File(sonidito);
        AudioFileFormat aff;
        AudioInputStream ais;
        try{
            aff=AudioSystem.getAudioFileFormat(sf);
            ais=AudioSystem.getAudioInputStream(sf);
            AudioFormat af=aff.getFormat();
            DataLine.Info info = new DataLine.Info(
                    Clip.class,
                    ais.getFormat(),
                    ((int) ais.getFrameLength() *af.getFrameSize()));
            Clip ol = (Clip) AudioSystem.getLine(info);
            ol.open(ais);
            ol.loop(0);
        } catch(UnsupportedAudioFileException ee){} catch(IOException ea){} catch(LineUnavailableException LUE){};
    }
    
    /**
     * Mtodo run de la interfaz runnable. Lee contnuamente por cambios en los usuarios y en los mensajes enviados.
     */
    
    public void run() {
        try {
            while (true) {
                String line = i.readUTF();
                if (line.substring(0,2).equals("->")) {
                    users.setText("\n");
                    users.append(line.substring(2));
                    sonido("Ring.wav");
                    
                    
                } else {
                    if (line.substring(0,2).equals("<-")) {
                        users.setText("\n");
                        users.append(line.substring(2));
                        sonido("ding.wav");
                    } else {
                        String[] split = line.split("<");
                        String veamos;
                        Integer intColor = new Integer(split[0]); // lo que esta antes del comienzo del user es el color!!!
                        Color color= new Color(intColor.intValue());
                        Integer hola= new Integer(color.getRGB());
                        veamos= Integer.toHexString(hola.intValue());
                        String[] split2 = split[1].split(">"); // extraigo el nombre de usuario.
                        
                        try{
                            archivo.seek(archivo.length());
                            archivo.writeUTF("<font color=\"#" + veamos.substring(2) + "\"> &lt; " + split2[0] +" &gt;" + split2[1] + "</font><br>\n"); // armo el mensaje en formato html.
                        } catch (IOException excepcionES){}
                        
                        try{
                            archivo.seek(0);
                            datos="";
                            while(true){
                                datos= datos + (String) archivo.readUTF();
                            }
                        } catch (IOException ex){}
                        
                        output.setText(datos);
                        output.setCaretPosition(output.getDocument().getLength()); //scroll al final
                    }
                }
            }
        } catch (IOException ex) {
            ex.printStackTrace();
        }
    }
    
    
}
